@@ -45,6 +45,7 @@ from flent import aggregators, formatters, resultset, loggers
 from flent.metadata import record_metadata, record_postrun_metadata
 from flent.util import clean_path, format_date, token_split
 from flent.settings import parser as SETTINGS_PARSER
+from flent.settings import _LOG_DEFER
 
 # Python2/3 compatibility
 try:
@@ -101,11 +102,12 @@ class BatchRunner(object):
             for name, vals in obj.items():
                 # Expand inheritance
                 if 'inherits' in vals:
-                    if not vals['inherits'] in obj:
-                        raise RuntimeError(
-                            "%s inherits from non-existent parent %s."
-                            % (name, vals['inherits']))
-                    obj[name] = self.inherit(obj[vals['inherits']], vals)
+                    for inh in [x.strip() for x in reversed(vals['inherits'].split(','))]:
+                        if not inh in obj:
+                            raise RuntimeError(
+                                "%s inherits from non-existent parent %s."
+                                % (name, inh))
+                        obj[name] = self.inherit(obj[inh], obj[name])
 
                 # Parse boolean options
                 for k, v in obj[name].items():
@@ -259,12 +261,11 @@ class BatchRunner(object):
                         raise
 
     def gen_filename(self, settings, batch, argset, rep):
-        filename = "batch-%s-%s-%s" % (
-            settings.BATCH_NAME,
-            batch['batch_time'],
-            batch.get('filename_extra', "%s-%s" % (argset, rep))
-        )
-        return clean_path(filename)
+        p = ["batch", settings.BATCH_NAME]
+        if self.settings.BATCH_TIMESTAMP:
+            p.append(batch['batch_time'])
+        p.append(batch.get('filename_extra', "%s-%s" % (argset, rep)))
+        return clean_path("-".join(p))
 
     def expand_argsets(self, batch, argsets, batch_time, batch_name,
                        print_status=True, no_shuffle=False):
@@ -496,6 +497,11 @@ class BatchRunner(object):
         settings = settings.copy()
         settings.load_test()
         res = resultset.new(settings)
+
+        if settings.LOG_FILE is _LOG_DEFER:
+            settings.LOG_FILE = res.dump_filename.replace(res.SUFFIX, ".log")
+            loggers.setup_logfile(settings.LOG_FILE)
+
         record_metadata(res, settings.EXTENDED_METADATA,
                         settings.REMOTE_METADATA)
 
@@ -513,7 +519,7 @@ class BatchRunner(object):
                                 settings.REMOTE_METADATA)
         res.dump_dir(output_path)
         logger.log(loggers.INFO if print_datafile_loc else loggers.DEBUG,
-                   "Data file written to %s.", res.dump_filename)
+                   "Data file written to %s", res.dump_filename)
 
         formatter = formatters.new(settings)
         formatter.format([res])
@@ -572,6 +578,7 @@ class BatchRunner(object):
         elif self.settings.BATCH_NAMES:
             start_time = self.settings.TIME
             self.settings.BATCH_UUID = str(uuid.uuid4())
+            self.settings.LOG_FILE = None  # batch run will generate logs
             logger.info("Started batch run %s at %s.",
                         self.settings.BATCH_UUID,
                         format_date(start_time, fmt="%Y-%m-%d %H:%M:%S"))
