@@ -38,7 +38,6 @@ from argparse import SUPPRESS
 from datetime import datetime
 from itertools import chain
 from multiprocessing import Pool, Queue
-from distutils.version import LooseVersion
 
 from flent import util, batch, loggers, resultset, plotters
 from flent.build_info import DATA_DIR, VERSION
@@ -47,16 +46,13 @@ from flent.loggers import get_logger, add_log_handler, remove_log_handler, \
 from flent.resultset import ResultSet
 from flent.settings import ListTests, new as new_settings, plot_group
 
+import matplotlib
+matplotlib.use("Agg")
+
 logger = get_logger(__name__)
 
 mswindows = (sys.platform == "win32")
 
-# Python 2/3 compatibility
-try:
-    unicode
-except NameError:
-    unicode = str
-
 try:
     from os import cpu_count
 except ImportError:
@@ -67,70 +63,34 @@ try:
 except NotImplementedError:
     CPU_COUNT = 1
 
-if plotters.MPL_VER < LooseVersion("1.4.2"):
-    raise RuntimeError("The GUI requires matplotlib version >= 1.4.2 to work.")
-else:
-    import matplotlib
-    matplotlib.use("Agg")
 
-FORCE_QT4 = bool(os.getenv("FORCE_QT4", False))
 try:
-    if FORCE_QT4:
-        raise ImportError("Force fallback to Qt4")
+    import qtpy
+    from qtpy import QtCore, QtGui, uic
 
-    from PyQt5 import QtCore, QtGui, uic
-
-    from PyQt5.QtWidgets import QMessageBox, QFileDialog, QTreeView, \
+    from qtpy.QtWidgets import QMessageBox, QFileDialog, QTreeView, \
         QAbstractItemView, QMenu, QAction, QTableView, QHeaderView, \
         QFormLayout, QHBoxLayout, QVBoxLayout, QApplication, QPlainTextEdit, \
         QWidget, QLineEdit, QComboBox, QSpinBox, QDoubleSpinBox, QScrollArea, \
-        QPushButton, QShortcut
+        QPushButton, QShortcut, QMainWindow, QDialog
 
-    from PyQt5.QtGui import QFont, QCursor, QMouseEvent, QKeySequence, \
-        QResizeEvent, QDesktopServices, QValidator
+    from qtpy.QtGui import QFont, QCursor, QMouseEvent, QKeySequence, \
+        QResizeEvent, QDesktopServices, QValidator, QGuiApplication
 
-    from PyQt5.QtCore import Qt, QIODevice, QByteArray, \
-        QDataStream, QSettings, QTimer, QEvent, pyqtSignal, \
+    from qtpy.QtCore import Qt, QIODevice, QByteArray, \
+        QDataStream, QSettings, QTimer, QEvent, Signal, \
         QAbstractItemModel, QAbstractTableModel, QModelIndex, \
         QItemSelectionModel, QStringListModel, QUrl
 
-    from PyQt5.QtNetwork import QLocalSocket, QLocalServer
+    from qtpy.QtNetwork import QLocalSocket, QLocalServer
 
     from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg \
         as FigureCanvas
     from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT \
         as NavigationToolbar
-    QTVER = 5
+
 except ImportError:
-    try:
-        from PyQt4 import QtCore, QtGui, uic
-
-        from PyQt4.QtGui import QMessageBox, QFileDialog, QTreeView, \
-            QAbstractItemView, QMenu, QAction, QFont, QTableView, QCursor, \
-            QHeaderView, QFormLayout, QHBoxLayout, QVBoxLayout, \
-            QItemSelectionModel, QMouseEvent, QApplication, QStringListModel, \
-            QKeySequence, QResizeEvent, QPlainTextEdit, QDesktopServices, \
-            QWidget, QLineEdit, QComboBox, QSpinBox, QDoubleSpinBox, QScrollArea,\
-            QPushButton, QValidator, QShortcut
-
-        from PyQt4.QtCore import Qt, QIODevice, QByteArray, \
-            QDataStream, QSettings, QTimer, QEvent, pyqtSignal, \
-            QAbstractItemModel, QAbstractTableModel, QModelIndex, QUrl
-
-        from PyQt4.QtNetwork import QLocalSocket, QLocalServer
-
-        from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg \
-            as FigureCanvas
-        from matplotlib.backends.backend_qt4agg import NavigationToolbar2QT \
-            as NavigationToolbar
-
-        QTVER = 4
-
-        if not FORCE_QT4:
-            logger.warning("Falling back to Qt4 for the GUI. "
-                           "Please consider installing PyQt5.\n")
-    except ImportError:
-        raise RuntimeError("Unable to find a usable PyQt version.")
+    raise RuntimeError("Unable to find a usable Qt version.")
 
 
 # The file selector dialog on OSX is buggy, so switching allowed file extensions
@@ -153,6 +113,9 @@ SOCKET_NAME_PREFIX = "flent-socket-"
 SOCKET_DIR = tempfile.gettempdir()
 WINDOW_STATE_VERSION = 1
 
+# Hack to propagate the --absolute-time option to multi-process helpers
+USE_ABSOLUTE_TIME = False
+
 ABOUT_TEXT = """<p>Flent version {version}.<br>
 Copyright &copy; 2017 Toke Høiland-Jørgensen and contributors.<br>
 Released under the GNU GPLv3.</p>
@@ -165,7 +128,7 @@ file an issue on Github<a>.</p>"""
 __all__ = ['run_gui']
 
 
-def run_gui(settings):
+def run_gui(settings, test_mode=False):
     if check_running(settings):
         return 0
 
@@ -180,10 +143,15 @@ def run_gui(settings):
     app = QApplication(sys.argv[:1])
     mainwindow = MainWindow(settings)
     mainwindow.show()
+    if test_mode:
+        mainwindow.defer_close()
     return app.exec_()
 
 
 def pool_init_func(settings, queue):
+    global USE_ABSOLUTE_TIME
+    USE_ABSOLUTE_TIME = settings.ABSOLUTE_TIME
+
     plotters.init_matplotlib("-", settings.USE_MARKERS,
                              settings.LOAD_MATPLOTLIBRC)
     set_queue_handler(queue)
@@ -238,24 +206,8 @@ def check_running(settings):
     return False
 
 
-def get_ui_class(filename):
-    """Helper method to dynamically load a .ui file, construct a class
-    inheriting from the ui class and the associated base class, and return
-    that constructed class.
-
-    This allows subclasses to inherit from the output of this function."""
-
-    try:
-        ui, base = uic.loadUiType(os.path.join(DATA_DIR, 'ui', filename))
-    except Exception as e:
-        raise RuntimeError("While loading ui file '%s': %s" % (filename, e))
-
-    class C(ui, base):
-
-        def __init__(self, *args):
-            base.__init__(self, *args)
-            self.setupUi(self)
-    return C
+def get_ui_file(filename):
+    return os.path.join(DATA_DIR, 'ui', filename)
 
 
 class LoadedResultset(dict):
@@ -264,7 +216,7 @@ class LoadedResultset(dict):
 
 def results_load_helper(filename):
     try:
-        r = ResultSet.load_file(filename)
+        r = ResultSet.load_file(filename, USE_ABSOLUTE_TIME)
         s = new_settings()
         s.update(r.meta())
         s.load_test(informational=True)
@@ -280,10 +232,11 @@ def results_load_helper(filename):
         return None
 
 
-class MainWindow(get_ui_class("mainwindow.ui")):
+class MainWindow(QMainWindow):
 
     def __init__(self, settings):
         super(MainWindow, self).__init__()
+        uic.loadUi(get_ui_file("mainwindow.ui"), self)
         self.settings = settings
         self.last_dir = os.getcwd()
 
@@ -330,9 +283,11 @@ class MainWindow(get_ui_class("mainwindow.ui")):
         self.expandButton.clicked.connect(self.metadata_column_resize)
 
         self.checkHighlight.setChecked(self.settings.HOVER_HIGHLIGHT)
-        self.checkDebugLog.setChecked(loggers.out_handler.level == loggers.DEBUG)
         self.checkExceptionLog.setChecked(self.settings.DEBUG_ERROR)
 
+        if loggers.out_handler:
+            self.checkDebugLog.setChecked(loggers.out_handler.level == loggers.DEBUG)
+
         self.checkHighlight.toggled.connect(self.update_checkboxes)
         self.checkDebugLog.toggled.connect(self.update_checkboxes)
         self.checkExceptionLog.toggled.connect(self.update_checkboxes)
@@ -390,7 +345,8 @@ class MainWindow(get_ui_class("mainwindow.ui")):
         QShortcut(QKeySequence("Ctrl+Up"),
                   self).activated.connect(self.prev_plot)
 
-        logger.info("GUI loaded. Running on PyQt v%s.", QtCore.PYQT_VERSION_STR)
+        logger.info("GUI loaded. Using Qt through %s v%s.", qtpy.API,
+                    QtCore.__version__)
 
     def read_log_queue(self):
         while not self.log_queue.empty():
@@ -471,7 +427,7 @@ class MainWindow(get_ui_class("mainwindow.ui")):
 
     def keyPressEvent(self, event):
         widget = self.viewArea.currentWidget()
-        text = unicode(event.text())
+        text = str(event.text())
         if widget and text in ('x', 'X', 'y', 'Y'):
             a = text.lower()
             d = 'in' if a == text else 'out'
@@ -547,7 +503,7 @@ class MainWindow(get_ui_class("mainwindow.ui")):
         if isinstance(filenames, tuple):
             filenames = filenames[0]
         if filenames:
-            self.last_dir = os.path.dirname(unicode(filenames[0]))
+            self.last_dir = os.path.dirname(str(filenames[0]))
 
         return filenames
 
@@ -634,6 +590,11 @@ class MainWindow(get_ui_class("mainwindow.ui")):
             self.load_files(self.defer_load)
             self.defer_load = None
 
+    def defer_close(self):
+        self.close_timer = QTimer(self)
+        self.close_timer.timeout.connect(self.close)
+        self.close_timer.start()
+
     def shorten_titles(self, titles):
         new_titles = []
         substr = util.long_substr(titles)
@@ -829,19 +790,21 @@ class MainWindow(get_ui_class("mainwindow.ui")):
 
         if isinstance(filenames[0], ResultSet):
             results = filenames
+            titles = self.shorten_titles([r.title for r in results])
         else:
             results = list(filter(None, self.worker_pool.map(results_load_helper,
-                                                             map(unicode,
+                                                             map(str,
                                                                  filenames))))
 
-        titles = self.shorten_titles([r['title'] for r in results])
+            titles = self.shorten_titles([r['title'] for r in results])
+
         self.focus_new = True
 
         self.load_queue.extend(zip(results, titles))
         self.load_timer.start()
 
         if set_last_dir:
-            self.last_dir = os.path.dirname(unicode(filenames[-1]))
+            self.last_dir = os.path.dirname(str(filenames[-1]))
 
     def load_one(self):
         if not self.load_queue:
@@ -910,20 +873,24 @@ class MainWindow(get_ui_class("mainwindow.ui")):
         dlg.exec_()
 
 
-class AboutDialog(get_ui_class("aboutdialog.ui")):
+class AboutDialog(QDialog):
 
     def __init__(self, parent):
         super(AboutDialog, self).__init__(parent)
+        uic.loadUi(get_ui_file("aboutdialog.ui"), self)
+
         self.aboutText.setText(ABOUT_TEXT.format(version=VERSION))
 
 
-class NewTestDialog(get_ui_class("newtestdialog.ui")):
+class NewTestDialog(QDialog):
 
     def __init__(self, parent, settings, log_queue):
         super(NewTestDialog, self).__init__(parent)
-        self.settings = settings.copy()
-        self.settings.INPUT = []
-        self.settings.GUI = False
+        uic.loadUi(get_ui_file("newtestdialog.ui"), self)
+        self.orig_settings = settings.copy()
+        self.orig_settings.INPUT = []
+        self.orig_settings.GUI = False
+        self.settings = self.orig_settings.copy()
         self.log_queue = log_queue
         self.pid = None
         self.aborted = False
@@ -994,13 +961,13 @@ class NewTestDialog(get_ui_class("newtestdialog.ui")):
             logger.error("Output directory does not exist.")
             return
 
-        test = unicode(test)
-        host = unicode(host)
-        path = unicode(path)
+        test = str(test)
+        host = str(host)
+        path = str(path)
 
-        self.settings.HOSTS = [host]
+        self.settings.HOSTS = util.token_split(host)
         self.settings.NAME = test
-        self.settings.TITLE = unicode(self.testTitle.text())
+        self.settings.TITLE = str(self.testTitle.text())
         self.settings.LENGTH = self.testLength.value()
         self.settings.DATA_DIR = path
         self.settings.EXTENDED_METADATA = self.extendedMetadata.isChecked()
@@ -1046,6 +1013,7 @@ class NewTestDialog(get_ui_class("newtestdialog.ui")):
         self.monitor_timer.stop()
         self.pid = None
         self.aborted = False
+        self.settings = self.orig_settings.copy()
 
     def keyPressEvent(self, evt):
         if evt.key() == Qt.Key_Escape:
@@ -1065,11 +1033,11 @@ class NewTestDialog(get_ui_class("newtestdialog.ui")):
                 elapsed = time.time() - self.start_time
                 self.progressBar.setValue(100 * elapsed / self.total_time)
         else:
-            self.reset()
             fn = os.path.join(self.settings.DATA_DIR,
                               self.settings.DATA_FILENAME)
             if os.path.exists(fn):
                 self.parent().load_files([fn])
+            self.reset()
 
 
 class QPlainTextLogger(loggers.Handler):
@@ -1176,7 +1144,7 @@ class MetadataModel(QAbstractItemModel):
         if idx.column() == 0:
             return item.name
         elif idx.column() == 1:
-            return unicode(item.value)
+            return str(item.value)
 
     def parent(self, idx):
         item = idx.internalPointer()
@@ -1210,12 +1178,18 @@ class MetadataView(QTreeView):
 
         def pin():
             self.add_pin(idx)
-        act_pin = QAction("&Pin expanded", menu, triggered=pin)
 
         def col():
             self.add_open_files_col(idx)
-        act_col = QAction("&Add open files column", menu, triggered=col)
-        menu.addActions([act_pin, act_col])
+
+        def copy():
+            self.copy_value(idx)
+
+        menu.addActions([
+            QAction("&Pin expanded", menu, triggered=pin),
+            QAction("&Add open files column", menu, triggered=col),
+            QAction("&Copy value to clipboard", menu, triggered=copy)
+        ])
         menu.exec_(event.globalPos())
         event.accept()
 
@@ -1243,6 +1217,13 @@ class MetadataView(QTreeView):
         self.openFilesView.horizontalHeader().add_column(None,
                                                          ":".join(map(str, path)))
 
+    def copy_value(self, idx):
+        val = self.model().data(self.model().index(idx.row(),
+                                                   1,
+                                                   idx.parent()),
+                                Qt.DisplayRole)
+        QGuiApplication.clipboard().setText(val)
+
     def setModel(self, model):
         super(MetadataView, self).setModel(model)
         self.restore_pinned()
@@ -1300,7 +1281,7 @@ class ActionWidget(object):
 
 class BooleanActionWidget(ActionWidget, QComboBox):
 
-    value_changed = pyqtSignal()
+    value_changed = Signal()
 
     def __init__(self, *args, **kwargs):
         super(BooleanActionWidget, self).__init__(*args, **kwargs)
@@ -1325,7 +1306,7 @@ class BooleanActionWidget(ActionWidget, QComboBox):
 
 class ChoicesActionWidget(ActionWidget, QComboBox):
 
-    value_changed = pyqtSignal()
+    value_changed = Signal()
 
     def __init__(self, *args, **kwargs):
         super(ChoicesActionWidget, self).__init__(*args, **kwargs)
@@ -1373,12 +1354,12 @@ class NoneSpinBoxMixin(object):
 
 
 class NoneDoubleSpinBox(NoneSpinBoxMixin, QDoubleSpinBox):
-    value_changed = pyqtSignal()
+    value_changed = Signal()
 
 
 class IntActionWidget(ActionWidget, NoneSpinBoxMixin, QSpinBox):
 
-    value_changed = pyqtSignal()
+    value_changed = Signal()
 
     def __init__(self, *args, **kwargs):
         super(IntActionWidget, self).__init__(*args, **kwargs)
@@ -1398,7 +1379,7 @@ class IntActionWidget(ActionWidget, NoneSpinBoxMixin, QSpinBox):
 
 class FloatActionWidget(ActionWidget, NoneSpinBoxMixin, QDoubleSpinBox):
 
-    value_changed = pyqtSignal()
+    value_changed = Signal()
 
     def __init__(self, *args, **kwargs):
         super(FloatActionWidget, self).__init__(*args, **kwargs)
@@ -1417,7 +1398,7 @@ class FloatActionWidget(ActionWidget, NoneSpinBoxMixin, QDoubleSpinBox):
 
 class PairActionWidget(ActionWidget, QWidget):
 
-    value_changed = pyqtSignal()
+    value_changed = Signal()
 
     def __init__(self, parent, action, widget=QLineEdit, **kwargs):
         super(PairActionWidget, self).__init__(parent, action, **kwargs)
@@ -1449,7 +1430,7 @@ class PairActionWidget(ActionWidget, QWidget):
 
 class FloatPairActionWidget(PairActionWidget):
 
-    value_changed = pyqtSignal()
+    value_changed = Signal()
 
     def __init__(self, *args, **kwargs):
         kwargs["widget"] = NoneDoubleSpinBox
@@ -1477,7 +1458,7 @@ class FloatPairActionWidget(PairActionWidget):
 
 class TextActionWidget(ActionWidget, QLineEdit):
 
-    value_changed = pyqtSignal()
+    value_changed = Signal()
 
     def __init__(self, *args, **kwargs):
         super(TextActionWidget, self).__init__(*args, **kwargs)
@@ -1498,9 +1479,9 @@ class TextActionWidget(ActionWidget, QLineEdit):
 
 class AddRemoveWidget(QWidget):
 
-    add_pressed = pyqtSignal()
-    remove_pressed = pyqtSignal('QWidget')
-    value_changed = pyqtSignal()
+    add_pressed = Signal()
+    remove_pressed = Signal('QWidget')
+    value_changed = Signal()
 
     def __init__(self, parent, subwidget):
         super(AddRemoveWidget, self).__init__(parent)
@@ -1536,7 +1517,7 @@ class AddRemoveWidget(QWidget):
 
 class MultiValWidget(ActionWidget, QWidget):
 
-    value_changed = pyqtSignal()
+    value_changed = Signal()
 
     def __init__(self, *args, **kwargs):
         widget = kwargs.get("widget", TextActionWidget)
@@ -1612,14 +1593,15 @@ class MultiValWidget(ActionWidget, QWidget):
 
 class SettingsWidget(QScrollArea):
 
-    values_changed = pyqtSignal()
+    values_changed = Signal()
 
     _widget_type_map = {int: IntActionWidget,
                         float: FloatActionWidget,
                         util.float_pair: FloatPairActionWidget,
+                        util.float_pair_noomit: FloatPairActionWidget,
                         util.comma_list: MultiValWidget,
                         util.keyval: PairActionWidget,
-                        unicode: TextActionWidget}
+                        str: TextActionWidget}
 
     def __init__(self, parent, options, settings, compact=False):
         super(SettingsWidget, self).__init__(parent)
@@ -1720,7 +1702,7 @@ class ResultsetStore(object):
 
         def get_key(itm):
             try:
-                return unicode(itm.meta(key))
+                return str(itm.meta(key))
             except KeyError:
                 return ''
         if only:
@@ -1875,7 +1857,7 @@ class OpenFilesModel(QAbstractTableModel):
 
     def get_metadata(self, idx, name):
         try:
-            return unicode(self.open_files[idx].meta(name))
+            return str(self.open_files[idx].meta(name))
         except KeyError:
             return None
 
@@ -2019,10 +2001,11 @@ class OpenFilesHeader(QHeaderView):
         event.accept()
 
 
-class AddColumnDialog(get_ui_class("addcolumn.ui")):
+class AddColumnDialog(QDialog):
 
     def __init__(self, parent, path=None):
         super(AddColumnDialog, self).__init__(parent)
+        uic.loadUi(get_ui_file("addcolumn.ui"), self)
 
         self.metadataPathEdit.textChanged.connect(self.update_name)
         self.columnNameEdit.textEdited.connect(self.name_entered)
@@ -2041,10 +2024,10 @@ class AddColumnDialog(get_ui_class("addcolumn.ui")):
         self.columnNameEdit.setText(parts[-1])
 
     def get_path(self):
-        return unicode(self.metadataPathEdit.text())
+        return str(self.metadataPathEdit.text())
 
     def get_name(self):
-        return unicode(self.columnNameEdit.text())
+        return str(self.columnNameEdit.text())
 
 
 class UpdateDisabler(object):
@@ -2069,17 +2052,18 @@ class FigureManager(matplotlib.backend_bases.FigureManagerBase):
         return self.widget.title
 
 
-class ResultWidget(get_ui_class("resultwidget.ui")):
+class ResultWidget(QWidget):
 
-    update_start = pyqtSignal()
-    update_end = pyqtSignal()
-    plot_changed = pyqtSignal('QString', 'QString')
-    new_plot = pyqtSignal()
-    name_changed = pyqtSignal()
+    update_start = Signal()
+    update_end = Signal()
+    plot_changed = Signal('QString', 'QString')
+    new_plot = Signal()
+    name_changed = Signal()
     default_title = "New tab"
 
     def __init__(self, parent, settings, worker_pool):
         super(ResultWidget, self).__init__(parent)
+        uic.loadUi(get_ui_file("resultwidget.ui"), self)
         self.results = None
         self.settings = settings.copy()
         self.dirty = True
@@ -2154,7 +2138,7 @@ class ResultWidget(get_ui_class("resultwidget.ui")):
         elif isinstance(results, ResultSet):
             self.results = results
         else:
-            self.results = ResultSet.load_file(unicode(results))
+            self.results = ResultSet.load_file(str(results))
             self.settings.compute_missing_results(self.results)
 
         if plot and plot in self.settings.PLOTS:
@@ -2193,7 +2177,7 @@ class ResultWidget(get_ui_class("resultwidget.ui")):
     def load_files(self, filenames):
         added = 0
         for f in filenames:
-            if self.add_extra(ResultSet.load_file(unicode(f))):
+            if self.add_extra(ResultSet.load_file(str(f))):
                 self.update(False)
                 added += 1
         self.redraw()
@@ -2265,7 +2249,7 @@ class ResultWidget(get_ui_class("resultwidget.ui")):
             return
         if isinstance(plot_name, QModelIndex):
             plot_name = self.plotModel.name_of(plot_name)
-        plot_name = unicode(plot_name)
+        plot_name = str(plot_name)
         if plot_name != self.settings.PLOT and plot_name in self.settings.PLOTS:
             self.settings.PLOT = plot_name
             self.plotSelectionModel.setCurrentIndex(
