@@ -30,6 +30,7 @@ from flent.loggers import get_logger
 
 logger = get_logger(__name__)
 
+TIMEOUT = 5
 INTERESTING_OFFLOADS = ['tcp-segmentation-offload',
                         'udp-fragmentation-offload',
                         'large-receive-offload',
@@ -41,6 +42,8 @@ INTERESTING_SYSCTLS = ['net.core.rmem_max',
                        'net.ipv4.tcp_autocorking',
                        'net.ipv4.tcp_early_retrans',
                        'net.ipv4.tcp_ecn',
+                       'net.ipv4.tcp_pacing_ca_ratio',
+                       'net.ipv4.tcp_pacing_ss_ratio',
                        'net.ipv4.tcp_dsack',
                        'net.ipv4.tcp_fack',
                        'net.ipv4.tcp_sack',
@@ -68,6 +71,11 @@ INTERESTING_MODULES = ['cake',
                        'mac80211',
                        'cfg80211']
 
+try:
+    processerror = subprocess.SubprocessError
+except AttributeError:
+    processerror = subprocess.CalledProcessError
+
 
 class CommandRunner(object):
 
@@ -96,11 +104,17 @@ class CommandRunner(object):
         try:
             if self.hostname:
                 command = "ssh %s '%s'" % (self.hostname, command)
-            res = subprocess.check_output(command, universal_newlines=True,
-                                          shell=True, stderr=subprocess.STDOUT,
-                                          env=self.env)
+            try:
+                res = subprocess.check_output(command, universal_newlines=True,
+                                              shell=True, stderr=subprocess.STDOUT,
+                                              env=self.env, timeout=TIMEOUT)
+            except TypeError:
+                # Python 2 doesn't have timeout arg
+                res = subprocess.check_output(command, universal_newlines=True,
+                                              shell=True, stderr=subprocess.STDOUT,
+                                              env=self.env)
             return res.strip()
-        except subprocess.CalledProcessError:
+        except processerror:
             return None
 
 
@@ -110,6 +124,7 @@ __all__ = ['record_metadata']
 
 
 def record_metadata(results, extended, hostnames):
+    logger.debug("Gathering local metadata")
     m = results.meta()
     get_command_output.set_hostname(None)
     m['KERNEL_NAME'] = get_command_output("uname -s")
@@ -123,10 +138,12 @@ def record_metadata(results, extended, hostnames):
     if extended:
         m['IP_ADDRS'] = get_ip_addrs()
         m['GATEWAYS'] = get_gateways()
+        m['WIFI_DATA'] = get_wifi_data()
 
     m['REMOTE_METADATA'] = {}
 
     for h in hostnames:
+        logger.debug("Gathering remote metadata from %s", h)
         get_command_output.set_hostname(h)
         m['REMOTE_METADATA'][h] = {}
         m['REMOTE_METADATA'][h]['LOCAL_HOST'] = get_command_output("hostname")
@@ -150,6 +167,7 @@ def record_metadata(results, extended, hostnames):
         if extended:
             m['REMOTE_METADATA'][h]['IP_ADDRS'] = get_ip_addrs()
             m['REMOTE_METADATA'][h]['GATEWAYS'] = get_gateways()
+            m['REMOTE_METADATA'][h]['WIFI_DATA'] = get_wifi_data()
 
 
 def record_postrun_metadata(results, extended, hostnames):
@@ -352,12 +370,12 @@ def get_egress_info(target, ip_version, extended):
         if 'nexthop' not in route:
             route['nexthop'] = None
 
-    if not extended:
-        for k in 'gateway', 'src', 'nexthop', 'target':
-            if k in route:
-                del route[k]
-        if route['link_params'] and 'ether' in route['link_params']:
-            del route['link_params']['ether']
+        if not extended:
+            for k in 'gateway', 'src', 'nexthop', 'target':
+                if k in route:
+                    del route[k]
+            if route['link_params'] and 'ether' in route['link_params']:
+                del route['link_params']['ether']
 
     return route or None
 
@@ -498,3 +516,53 @@ def get_module_versions():
             module_versions[m] = v
 
     return module_versions
+
+
+def get_wifi_data():
+    wifi_data = {}
+
+    unwanted_keys = ["Interface", "ifindex", "wdev", "wiphy"]
+
+    output = get_command_output("iw dev")
+    iface = None
+    if output is not None:
+
+        for line in output.splitlines():
+
+            parts = line.split()
+            if len(parts) < 2:
+                continue
+
+            k, v = parts[0], parts[1]
+
+            if k == 'Interface':
+                iface = v
+                wifi_data[iface] = {}
+                continue
+            elif iface is None:
+                continue
+
+            if k in unwanted_keys:
+                continue
+
+            if k == 'txpower':
+                v = float(v)
+
+            if k == 'channel':
+                # This condition will return a dict with all the values of the channel
+                # With the input "channel 1 (2412 MHz), width: 20 MHz, center1: 2412 MHz"
+                # the output will be {'addr':..., channel': {'band': 2462, 'center1': 2462, 'number': 11, 'width': 20}, 'ssid':...}
+                v = {}
+                v['number'] = int(parts[1])
+                v['band'] = int(parts[2].strip("("))
+                v['width'] = int(parts[5])
+                v['center1'] = int(parts[8])
+
+            if line.strip() == "multicast TXQ:":
+                # No interesting output after this
+                iface = None
+                continue
+
+            wifi_data[iface][k] = v
+
+    return wifi_data
