@@ -23,12 +23,14 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 
 import io
 import math
+import os
 import re
 import warnings
 
-from flent import combiners, transformers
+from flent import combiners
 from flent.util import classname, long_substr, format_date, diff_parts, \
-    Glob, Update, float_pair, keyval, comma_list, ArgParam, ArgParser
+    Glob, Update, float_pair, float_pair_noomit, keyval, comma_list, ArgParam, \
+    ArgParser
 from flent.build_info import VERSION
 from flent.loggers import get_logger
 
@@ -36,23 +38,23 @@ from argparse import SUPPRESS
 from functools import reduce
 from itertools import cycle, islice, chain
 from collections import OrderedDict
+from distutils.version import LooseVersion
+
+logger = get_logger(__name__)
 
 try:
     import matplotlib
     import numpy as np
     HAS_MATPLOTLIB = True
-except ImportError:
+    MPL_VER = LooseVersion(matplotlib.__version__)
+    if MPL_VER < LooseVersion("1.5"):
+        logger.warning("Cannot use old matplotlib version %s, please upgrade!",
+                       matplotlib.__version__)
+        raise ImportError("Matplotlib %s too old" % matplotlib.__version__)
+except ImportError as e:
+    logger.debug("Unable to import matplotlib: %s", e)
     HAS_MATPLOTLIB = False
-
-# Python 2/3 compatibility
-try:
-    unicode
-    PY2 = True
-except NameError:
-    unicode = str
-    PY2 = False
-
-logger = get_logger(__name__)
+    MPL_VER = LooseVersion("0")
 
 PLOT_KWARGS = (
     'alpha',
@@ -122,28 +124,6 @@ MATPLOTLIB_STYLES = {'axes.axisbelow': True,
 
 MATPLOTLIB_INIT = False
 
-if PY2:
-    # Matplotlib will tend to pass the values directly to backends where they
-    # will be converted into native types. This breaks on some versions of
-    # matplotlib running on Python 2 if the values are unicode objects (which
-    # can't always be automatically converted). To work around this, encode
-    # everything if we are running on Python 2.
-
-    def filt(x):
-        try:
-            return x.encode()
-        except AttributeError:
-            return x
-
-    LINESTYLES = list(map(filt, LINESTYLES))
-    MARKERS = list(map(filt, MARKERS))
-    COLOURS = list(map(filt, COLOURS))
-
-    for k, v in MATPLOTLIB_STYLES.items():
-        MATPLOTLIB_STYLES[k] = filt(v)
-
-    del filt
-
 
 def init_matplotlib(output, use_markers, load_rc):
     if not HAS_MATPLOTLIB:
@@ -171,6 +151,9 @@ def init_matplotlib(output, use_markers, load_rc):
             raise RuntimeError(
                 "Unrecognised file format for output '%s'" % output)
 
+    elif not os.getenv("DISPLAY"):
+        matplotlib.use("agg")
+
     from matplotlib import pyplot
 
     for ls in LINESTYLES:
@@ -249,13 +232,13 @@ def add_plotting_args(parser):
 
     parser.add_argument(
         "--label-x",
-        action="append", dest="LABEL_X", type=unicode, default=[],
+        action="append", dest="LABEL_X", type=str, default=[],
         help="Override X axis labels. "
         "Can be specified twice, corresponding to figures with multiple axes.")
 
     parser.add_argument(
         "--label-y",
-        action="append", dest="LABEL_Y", type=unicode, default=[],
+        action="append", dest="LABEL_Y", type=str, default=[],
         help="Override Y axis labels. "
         "Can be specified twice, corresponding to figures with multiple axes.")
 
@@ -277,7 +260,7 @@ def add_plotting_args(parser):
                         const="log10", help=SUPPRESS)
     parser.add_argument(
         "--log-scale-y",
-        action="store", type=unicode, dest="LOG_SCALE", choices=("log2", "log10"),
+        action="store", type=str, dest="LOG_SCALE", choices=("log2", "log10"),
         help="Use logarithmic scale.")
 
     parser.add_argument(
@@ -287,6 +270,13 @@ def add_plotting_args(parser):
         "points by this value. Can be specified multiple times, in which case "
         "each value corresponds to a data series.")
 
+    parser.add_argument(
+        "--data-cutoff",
+        action="store", type=float_pair_noomit, dest="DATA_CUTOFF",
+        help="Data cutoff interval. Cut off all data points outside this "
+        "interval before plotting. For aggregate plots, this will happen "
+        "*before* aggregation, so for instance mean values will be affected.")
+
     parser.add_argument(
         "--bounds-x",
         action="append", dest="BOUNDS_X", type=float_pair, default=[],
@@ -357,7 +347,7 @@ def add_plotting_args(parser):
 
     parser.add_argument(
         "--figure-note", "--fig-note",
-        action="store", type=unicode, dest="FIG_NOTE",
+        action="store", type=str, dest="FIG_NOTE",
         help="Figure note. Will be added to the bottom-left corner of the "
         "figure.")
 
@@ -368,7 +358,7 @@ def add_plotting_args(parser):
 
     parser.add_argument(
         "--override-title",
-        action="store", type=unicode, dest="OVERRIDE_TITLE", metavar="TITLE",
+        action="store", type=str, dest="OVERRIDE_TITLE", metavar="TITLE",
         help="Override plot title. This parameter takes "
         "precedence over --no-title.")
 
@@ -402,12 +392,12 @@ def add_plotting_args(parser):
 
     parser.add_argument(
         "--legend-title",
-        action="store", type=unicode, dest="LEGEND_TITLE",
+        action="store", type=str, dest="LEGEND_TITLE",
         help="Override legend title.")
 
     parser.add_argument(
         "--legend-placement",
-        action="store", type=unicode, dest="LEGEND_PLACEMENT",
+        action="store", type=str, dest="LEGEND_PLACEMENT",
         choices=('best',
                  'upper right',
                  'upper left',
@@ -449,32 +439,32 @@ def add_plotting_args(parser):
 
     parser.add_argument(
         "--filter-regexp",
-        action="append", type=unicode, dest="FILTER_REGEXP", metavar="REGEXP",
+        action="append", type=str, dest="FILTER_REGEXP", metavar="REGEXP",
         default=[], help="Filter labels (regex). Filter out supplied regular "
         "expression from label names. Can be specified multiple times, in which "
         "case the regular expressions will be filtered in the order specified.")
 
     parser.add_argument(
         "--override-label",
-        action="append", type=unicode, dest="OVERRIDE_LABELS", metavar="LABEL",
+        action="append", type=str, dest="OVERRIDE_LABELS", metavar="LABEL",
         default=[],
         help="Override dataset labels. Must be specified multiple times "
         "corresponding to the datasets being overridden.")
 
     parser.add_argument(
         "--filter-series",
-        action="append", type=unicode, dest="FILTER_SERIES", metavar="SERIES",
+        action="append", type=str, dest="FILTER_SERIES", metavar="SERIES",
         default=[], help="Filter (hide) data series. Filters out specified "
         "series names from the plot. Can be specified multiple times.")
 
     parser.add_argument(
         "--split-group",
-        action="append", type=unicode, dest="SPLIT_GROUPS", default=[],
+        action="append", type=str, dest="SPLIT_GROUPS", default=[],
         metavar="LABEL",
-        help="New groups for box plots. Specify this option multiple "
+        help="New groups for box and bar plots. Specify this option multiple "
         "times to define the new groups that data sets should be split into on "
-        "box plots. The value of each option is the group name (displayed at the "
-        "top of the plot).")
+        "box and bar plots. The value of each option is the group name "
+        "(displayed at the top of the plot).")
 
     parser.add_argument(
         "--colours",
@@ -482,16 +472,25 @@ def add_plotting_args(parser):
         help="Override plot colours. Specify a comma-separated list of colours "
         "to be used for the plot colour cycle.")
 
+    parser.add_argument(
+        "--override-colour-mode",
+        action="store", type=str, dest="OVERRIDE_COLOUR_MODE", metavar="MODE",
+        help="Override colour_mode attribute. This changes the way colours "
+        "are assigned to bar plots. The default is 'groups' which assigns a "
+        "separate colour to each group of data series. The alternative is "
+        "'series' which assigns a separate colour to each series, repeating them"
+        "for each data group.")
+
     parser.add_argument(
         "--override-group-by",
-        action="store", type=unicode, dest="OVERRIDE_GROUP_BY", metavar="GROUP",
+        action="store", type=str, dest="OVERRIDE_GROUP_BY", metavar="GROUP",
         help="Override group_by attribute. This changes the way combination "
         "plots are created by overriding the function that is used to combine "
         "several data series into one.")
 
     hide_gui(parser.add_argument(
         "--combine-save-dir",
-        action="store", type=unicode, dest="COMBINE_SAVE_DIR",
+        action="store", type=str, dest="COMBINE_SAVE_DIR",
         metavar="DIRNAME",
         help="Save intermediate combination data. When doing a combination plot "
         "save the intermediate data to DIRNAME. This can then be used for "
@@ -540,7 +539,7 @@ def add_plotting_args(parser):
 
     hide_gui(parser.add_argument(
         "--scale-data",
-        action="append", type=unicode, dest="SCALE_DATA", default=[],
+        action="append", type=str, dest="SCALE_DATA", default=[],
         help="Extra scale data. Additional data files to consider when scaling "
         "the plot axes "
         "(for plotting several plots with identical axes). Note, this displays "
@@ -561,6 +560,7 @@ def new(settings, plotter=None, in_worker=False, **kwargs):
             data_config=settings.DATA_SETS,
             output=settings.OUTPUT,
             gui=settings.GUI,
+            absolute_time=settings.ABSOLUTE_TIME,
             description=settings.DESCRIPTION,
             in_worker=in_worker,
 
@@ -605,6 +605,7 @@ class Plotter(ArgParam):
                  figure=None,
                  results=None,
                  in_worker=False,
+                 absolute_time=False,
                  **kwargs):
         super(Plotter, self).__init__(**kwargs)
 
@@ -621,6 +622,8 @@ class Plotter(ArgParam):
         self.metadata = None
         self.callbacks = []
         self.in_worker = in_worker
+        self.combined = False
+        self.absolute_time = absolute_time
 
         self.gui = gui
         self.description = description
@@ -660,20 +663,6 @@ class Plotter(ArgParam):
 
     def __getstate__(self):
         state = self.__dict__.copy()
-
-        for c in state['configs']:
-            for a in c['axes']:
-                # There is a bug in matplotlib prior to 1.4.2 where the renderer
-                # can't be pickled. Work around this by monkey-patching the
-                # renderer object as needed.
-                if a._cachedRenderer and not hasattr(a._cachedRenderer,
-                                                     "__getstate__"):
-                    def getState():
-                        return {'width': a._cachedRenderer.width,
-                                'height': a._cachedRenderer.height,
-                                'dpi': a._cachedRenderer.dpi}
-                    a._cachedRenderer.__getstate__ = getState
-
         return state
 
     def init(self, config=None, axis=None):
@@ -682,11 +671,8 @@ class Plotter(ArgParam):
         self.configs = [self.config]
 
     def verify(self):
-        for a in self.figure.axes:
-            if len(a.get_lines()) > 0:
-                return True
-
-        return False
+        lengths = [len(a.get_lines()) for a in self.figure.axes]
+        return any(lengths), lengths
 
     def axes_iter(self):
         return iter(reduce(lambda x, y: x + y, [i['axes'] for i in self.configs]))
@@ -701,8 +687,11 @@ class Plotter(ArgParam):
                 for d in Glob.expand_list(s['data'], data.keys()):
                     if 'label' in s:
                         d_id = data[d]['id'] if 'id' in data[d] else d
-                        ns.append(dict(s, data=d, id=d_id,
-                                       label='%s -- %s' % (s['label'], d_id)))
+                        if s['label']:
+                            lbl = '%s -- %s' % (s['label'], d_id)
+                        else:
+                            lbl = d_id
+                        ns.append(dict(s, data=d, id=d_id, label=lbl))
                         if 'parent_id' in data[d]:
                             ns[-1]['parent_id'] = data[d]['parent_id']
                     else:
@@ -723,6 +712,8 @@ class Plotter(ArgParam):
                                             set()))
                     rks = Glob.expand_list(s['raw_key'], all_rks)
                     for k, n in zip(rks, diff_parts(rks, "::")):
+                        if k in self.filter_series:
+                            continue
                         if 'label' in s:
                             nns.append(dict(s, raw_key=k,
                                             label="%s -- %s" % (s['label'], n)))
@@ -745,6 +736,7 @@ class Plotter(ArgParam):
             for l, s in zip(self.override_labels, new_series):
                 if l is not None:
                     s['label'] = l
+                    s['label_override'] = True
 
         return dict(config, series=new_series)
 
@@ -792,7 +784,7 @@ class Plotter(ArgParam):
         # appropriate output format based on the file name.
         if self.output == "-":
             self.size_legends()
-            if not self.gui:
+            if not self.gui and not self.in_worker:
                 logger.debug("Showing matplotlib pyplot viewer")
                 pyplot.show()
         else:
@@ -1040,13 +1032,19 @@ class Plotter(ArgParam):
            and not self.horizontal_legend \
            and not self.legend_placement \
            and self.legends:
-            legend_width = max(
-                [l.get_window_extent().width for l in self.legends])
 
-            if not legend_width:  # Legend width is not set before it's drawn
+            # Make sure we have a renderer to get size from
+            renderer = getattr(self.figure, '_cachedRenderer', None)
+            if not renderer:
                 self.figure.canvas.draw()
+                renderer = getattr(self.figure, '_cachedRenderer', None)
+
+            try:
                 legend_width = max(
-                    [l.get_window_extent().width for l in self.legends])
+                    [l.get_window_extent(renderer).width for l in self.legends])
+            except Exception as e:
+                logger.debug("Error getting legend sizes: %s", e)
+                return
 
             canvas_width = self.figure.canvas.get_width_height()[0]
             for a in self.axes_iter():
@@ -1092,19 +1090,21 @@ class Plotter(ArgParam):
                                     format_date(self.metadata['TIME']),
                                     self.metadata['LENGTH'],
                                     self.metadata['STEP_SIZE'])
-            self.btm_art.append(self.figure.text(0.5,
-                                                 0.04 / self.figure.get_figheight(),
-                                                 annotation_string,
-                                                 horizontalalignment='center',
-                                                 verticalalignment='bottom',
-                                                 fontsize=8))
+            self.btm_art.append(
+                self.figure.text(0.5,
+                                 0.04 / self.figure.get_figheight(),
+                                 annotation_string,
+                                 horizontalalignment='center',
+                                 verticalalignment='bottom',
+                                 fontsize=8))
         if self.fig_note:
-            self.btm_art.append(self.figure.text(0.0,
-                                                 0.04 / self.figure.get_figheight(),
-                                                 self.fig_note,
-                                                 horizontalalignment='left',
-                                                 verticalalignment='bottom',
-                                                 fontsize=8))
+            self.btm_art.append(
+                self.figure.text(0.0,
+                                 0.04 / self.figure.get_figheight(),
+                                 self.fig_note,
+                                 horizontalalignment='left',
+                                 verticalalignment='bottom',
+                                 fontsize=8))
 
         return titles
 
@@ -1220,13 +1220,13 @@ class Plotter(ArgParam):
             # of the *axis* range, not the data range (this may be a big
             # difference if the data range is small).
             top_scale = top_percentile * 1.01
-            axis.set_ylim(ymin=0, ymax=top_scale)
+            axis.set_ylim(0, top_scale)
         else:
             if self.log_base:
                 axis.set_yscale('log', basey=self.log_base)
-                axis.set_ylim(ymin=max(0, btm_scale), ymax=top_scale)
+                axis.set_ylim(max(0, btm_scale), top_scale)
             else:
-                axis.set_ylim(ymin=btm_scale, ymax=top_scale)
+                axis.set_ylim(btm_scale, top_scale)
 
         if self.invert_y and unit in self.inverted_units:
             axis.invert_yaxis()
@@ -1239,16 +1239,10 @@ class Plotter(ArgParam):
             ma = np.ma.masked_invalid(arr)
             return np.percentile(ma.compressed(), q)
 
-    def _transform_data(self, data, t):
-        t = t.strip()
-        if hasattr(transformers, t):
-            data = getattr(transformers, t)(data)
-        return data
-
     def get_series(self, series, results, config,
                    no_invalid=False, aligned=False):
 
-        if aligned:
+        if aligned or self.combined:
             data = np.array((results.x_values,
                              results.series(series['data'])),
                             dtype=float)
@@ -1257,26 +1251,13 @@ class Plotter(ArgParam):
             try:
 
                 data = np.array(
-                    list(results.raw_series(series['data'], raw_key=raw_key)),
+                    list(results.raw_series(series['data'], raw_key=raw_key,
+                                            absolute=self.absolute_time)),
                     dtype=float).transpose()
 
                 if not len(data):
                     raise KeyError()
 
-                data_transform = series.get('data_transform')
-
-                if not data_transform and 'FAKE_RAW_VALUES' not in results.meta():
-                    cfgn = "%s::%s" % (series['data'], raw_key)
-                    if cfgn not in self.data_config:
-                        cfgn = series['data']
-
-                    try:
-                        data_transform = self.data_config[cfgn]['data_transform']
-                    except KeyError:
-                        pass
-
-                if data_transform:
-                    data[1] = self._transform_data(data[1], data_transform)
             except KeyError:
                 if raw_key:
                     # No point in using synthesised results since those won't be
@@ -1288,12 +1269,16 @@ class Plotter(ArgParam):
                                  results.series(series['data'])),
                                 dtype=float)
 
-        if 'cutoff' in config and config['cutoff'] and data.any():
-            min_x = data[0].min() + config['cutoff'][0]
-            max_x = data[0].max() + config['cutoff'][1]
+        if data.any() and (self.data_cutoff or config.get('cutoff')):
+            start, end = self.data_cutoff or config['cutoff']
+            if self.absolute_time:
+                start += results.t0
 
-            min_idx = data[0].searchsorted(min_x, side='right')
-            max_idx = data[0].searchsorted(max_x, side='left')
+            if end <= 0:
+                end += results.meta("TOTAL_LENGTH")
+
+            min_idx = data[0].searchsorted(start, side='left')
+            max_idx = data[0].searchsorted(end, side='right')
 
             data = data[:, min_idx:max_idx]
 
@@ -1306,19 +1291,28 @@ class Plotter(ArgParam):
         if 'norm_factor' in series:
             data[1] /= series['norm_factor']
 
-        if 'smoothing' in series and series['smoothing']:
+        if 'smoothing' in series and series['smoothing'] and data.any():
             l = series['smoothing']
             if l % 2 != 1:
                 l += 1
-            kern = np.ones(l, dtype=float)
-            kern /= l
-            data[1] = np.convolve(data[1], kern, mode=str('same'))
+
+            if l <= len(data[1]):
+                kern = np.ones(l, dtype=float)
+                kern /= l
+                data[1] = np.convolve(data[1], kern, mode=str('same'))
+            else:
+                logger.warn("Smoothing length longer than data series %s; "
+                            "not smoothing", series['data'])
 
         return data
 
 
 class CombineManyPlotter(object):
 
+    def __init__(self, *args, **kwargs):
+        super(CombineManyPlotter, self).__init__(*args, **kwargs)
+        self.combined = True
+
     def plot(self, results, config=None, axis=None, connect_interactive=True):
         if self.metadata is None:
             self.metadata = results[0].meta()
@@ -1329,7 +1323,8 @@ class CombineManyPlotter(object):
         combiner = combiners.new(combine_mode, print_n=self.combine_print_n,
                                  filter_regexps=self.filter_regexp,
                                  filter_series=self.filter_series,
-                                 save_dir=self.combine_save_dir)
+                                 save_dir=self.combine_save_dir,
+                                 data_cutoff=self.data_cutoff)
         super(CombineManyPlotter, self).plot(
             combiner(results, config),
             config,
@@ -1373,7 +1368,7 @@ class TimeseriesPlotter(Plotter):
             else:
                 a = 0
             s_unit = self.data_config[s['data']]['units']
-            if unit[a] is not None and s_unit != unit[a]:
+            if unit[a] is not None and s_unit != unit[a] and 'raw_key' not in s:
                 raise RuntimeError(
                     "Plot axis unit mismatch: %s/%s" % (unit[a], s_unit))
             unit[a] = s_unit
@@ -1405,7 +1400,8 @@ class TimeseriesPlotter(Plotter):
 
         stack = 'stacked' in config and config['stacked']
 
-        x_min = x_max = 0
+        x_min = 2**63
+        x_max = 0
 
         colours = cycle(self.colours)
 
@@ -1524,17 +1520,7 @@ class BoxPlotter(TimeseriesPlotter):
             self.metadata = results[0].meta()
         return self._plot(results, config, axis)
 
-    def _plot(self, results, config=None, axis=None):
-        if config is None:
-            config = self.config
-        axis = config['axes'][0]
-
-        ticklabels = []
-        ticks = []
-        texts = []
-        pos = 1
-        all_data = [None] * len(config['axes'])
-
+    def _get_split_groups(self, results, config):
         if self.split_groups:
             if len(results) % len(self.split_groups) > 0:
                 raise RuntimeError(
@@ -1554,12 +1540,29 @@ class BoxPlotter(TimeseriesPlotter):
             split_results = []
             series = config['series']
 
+        return group_size, split_results, series
+
+    def _plot(self, results, config=None, axis=None):
+        if config is None:
+            config = self.config
+        axis = config['axes'][0]
+
+        ticklabels = []
+        ticks = []
+        texts = []
+        pos = 1
+        all_data = [None] * len(config['axes'])
+
+        group_size, split_results, series = self._get_split_groups(results,
+                                                                   config)
+
         # The median lines are red, so filter out red from the list of colours
         colours = list(
             islice(cycle([c for c in self.colours if c != 'r']), group_size))
-        series_labels = self._filter_labels(
-            [s['label'] for s in series])
-
+        series_labels = [s.get('label', '') for s in series]
+        if not any([s.get('label_override', False) for s in series]):
+            series_labels = self._filter_labels(series_labels)
+        ticklabel_override = False
         for i, s in enumerate(series):
             if split_results:
                 results = split_results[i]
@@ -1569,6 +1572,7 @@ class BoxPlotter(TimeseriesPlotter):
                 a = 0
 
             data = []
+            group_size = len(results)
             for r in results:
                 d = self.get_series(s, r, config, no_invalid=True)
 
@@ -1585,6 +1589,8 @@ class BoxPlotter(TimeseriesPlotter):
             if not data:
                 continue
 
+            group_size = len(data)
+
             if len(series) > 1 or self.print_title:
                 texts.append(config['axes'][0].text(
                     pos + group_size / 2.0 - 0.5,
@@ -1597,12 +1603,15 @@ class BoxPlotter(TimeseriesPlotter):
 
             bp = config['axes'][a].boxplot(data,
                                            positions=positions, sym="b+")
-            for j, r in enumerate(results):
+            for j, r in zip(range(group_size), results):
                 pyplot.setp(bp['boxes'][j], color=colours[j])
                 if i == 0 and group_size > 1:
                     bp['caps'][j * 2].set_label(r.label())
                 if len(results) > 1:
                     ticklabels.append(r.label())
+                    ticklabel_override = (ticklabel_override or
+                                          r.metadata.get('label_override',
+                                                         False))
                 if len(bp['fliers']) == group_size:
                     pyplot.setp([bp['fliers'][j]], markeredgecolor=colours[j])
                     keys = 'caps', 'whiskers'
@@ -1620,13 +1629,20 @@ class BoxPlotter(TimeseriesPlotter):
             config['axes'][a].axvline(
                 x=pos + group_size, color='black', linewidth=0.5, linestyle=':')
             pos += group_size + 1
+
+        if not ticks:
+            return  # no data
+
         for i, a in enumerate(config['axes']):
-            self._do_scaling(a, all_data[i], 0, 100,
-                             config['units'][i], allow_log=False)
+            if all_data[i] is not None:
+                self._do_scaling(a, all_data[i], 0, 100,
+                                 config['units'][i], allow_log=False)
 
         for a, b in zip(config['axes'], self.bounds_y):
             a.set_ybound(b)
 
+        if not ticklabel_override:
+            ticklabels = self._filter_labels(ticklabels)
         for i, l in enumerate(ticklabels):
             if len(l) > self._max_label_length:
                 ticklabels[i] = l[:self._max_label_length] + "..."
@@ -1674,18 +1690,24 @@ class BarPlotter(BoxPlotter):
         errcol = 'k'
         width = 1.0
 
+        group_size, split_results, series = self._get_split_groups(results,
+                                                                   config)
         # The error bars lines are black, so filter out black from the list of
         # colours
         colours = list(
             islice(cycle([c for c in self.colours if c != errcol]),
-                   len(config['series'])))
+                   max(group_size, len(series))))
+
+        colour_mode = (self.override_colour_mode or
+                       config.get('colour_mode', 'groups'))
 
-        labels = self._filter_labels([r.label() for r in results])
         series_labels = self._filter_labels(
-            [s['label'] for s in config['series']])
+            [s['label'] for s in series])
         texts = []
 
-        for i, s in enumerate(config['series']):
+        for i, s in enumerate(series):
+            if split_results:
+                results = split_results[i]
             if 'axis' in s and s['axis'] == 2:
                 a = 1
             else:
@@ -1706,12 +1728,13 @@ class BarPlotter(BoxPlotter):
                     all_data[a].append(data[-1] + errors[-1])
                     all_data[a].append(data[-1] - errors[-1])
 
+            # may have skipped series, recalculate
             group_size = len(data)
 
             positions = [p - width / 2.0 for p in range(pos, pos + group_size)]
             ticks.extend(list(range(pos, pos + group_size)))
-            ticklabels.extend(labels)
-            if config.get('colour_by', 'groups') == 'groups':
+            ticklabels.extend(self._filter_labels([r.label() for r in results]))
+            if colour_mode == 'groups':
                 colour = colours[i]
             else:
                 colour = self.colours[:len(data)]
@@ -1730,6 +1753,9 @@ class BarPlotter(BoxPlotter):
                 x=pos + group_size, color='black', linewidth=0.5, linestyle=':')
             pos += group_size + 1
 
+        if not ticks:
+            return  # no data
+
         for a, b in zip(config['axes'], self.bounds_y):
             a.set_ybound(b)
 
@@ -1743,7 +1769,6 @@ class BarPlotter(BoxPlotter):
             if len(l) > self._max_label_length:
                 ticklabels[i] = l[:self._max_label_length] + "..."
 
-        axis.set_xticklabels(ticklabels, rotation=90, ha='center')
         axis.set_xlim(0, pos - 1)
         axis.set_xticks(ticks)
         axis.set_xticks([], minor=True)
